// src/lib/db.ts
import { openDB, type IDBPDatabase, type DBSchema } from 'idb';
import type { ListDetail, ListPublic } from './schemas/list'; // Import your list types
import type { ItemPublic } from './schemas/item'; // Import your item type

const DB_NAME = 'SharedListsDB';
const DB_VERSION = 1; // Increment this when changing schema

// Define the structure for queued actions
export interface SyncAction {
    id?: number; // Optional: will be added by IndexedDB autoIncrement
    type: 'create_list' | 'update_list' | 'delete_list' | 'create_item' | 'update_item' | 'delete_item';
    payload: any; // Data needed for the API call (e.g., listId, itemId, updateData)
    timestamp: number;
    tempId?: string; // Optional temporary ID for optimistic UI mapping (e.g., for newly created items)
}

// Define the database schema using TypeScript interface
interface SharedListsDBSchema extends DBSchema {
    lists: {
        key: number; // Primary key (list.id)
        value: ListDetail; // Store full detail including items
        indexes: Record<string, string>; // Example indexes
    };
    items: {
        key: number; // Primary key (item.id)
        value: ItemPublic;
        indexes: Record<string, string>; // Index by listId is crucial
    };
    syncQueue: {
        key: number; // Auto-incrementing key
        value: SyncAction;
        // No indexes needed for simple queue processing
    };
}

let dbPromise: Promise<IDBPDatabase<SharedListsDBSchema>> | null = null;

/** Gets the IndexedDB database instance, creating/upgrading if necessary. */
function getDb(): Promise<IDBPDatabase<SharedListsDBSchema>> {
    if (!dbPromise) {
        dbPromise = openDB<SharedListsDBSchema>(DB_NAME, DB_VERSION, {
            upgrade(db, oldVersion, newVersion, transaction, event) {
                console.log(`Upgrading DB from version ${oldVersion} to ${newVersion}`);

                // Create 'lists' store if it doesn't exist
                if (!db.objectStoreNames.contains('lists')) {
                    const listStore = db.createObjectStore('lists', { keyPath: 'id' });
                    listStore.createIndex('groupId', 'group_id'); // Index for potential filtering by group
                    listStore.createIndex('updated_at', 'updated_at'); // Index for sorting/filtering by date
                    console.log('Created lists object store');
                }

                // Create 'items' store if it doesn't exist
                if (!db.objectStoreNames.contains('items')) {
                    const itemStore = db.createObjectStore('items', { keyPath: 'id' });
                    // Crucial index for fetching items belonging to a list
                    itemStore.createIndex('listId', 'list_id');
                    itemStore.createIndex('updated_at', 'updated_at'); // Index for sorting/filtering by date
                    console.log('Created items object store');
                }

                // Create 'syncQueue' store if it doesn't exist
                if (!db.objectStoreNames.contains('syncQueue')) {
                    // Use autoIncrementing key
                    db.createObjectStore('syncQueue', { autoIncrement: true, keyPath: 'id' });
                    console.log('Created syncQueue object store');
                }

                // --- Handle specific version upgrades ---
                // Example: If upgrading from version 1 to 2
                // if (oldVersion < 2) {
                //   // Make changes needed for version 2
                //   const listStore = transaction.objectStore('lists');
                //   // listStore.createIndex('newIndex', 'newField');
                // }
                // if (oldVersion < 3) { ... }
            },
            blocked(currentVersion, blockedVersion, event) {
                // Fires if an older version of the DB is open in another tab/window
                console.error(`IndexedDB blocked. Current: ${currentVersion}, Blocked: ${blockedVersion}. Close other tabs.`);
                alert('Database update blocked. Please close other tabs/windows using this app and refresh.');
            },
            blocking(currentVersion, blockedVersion, event) {
                // Fires in the older tab/window that is blocking the upgrade
                console.warn(`IndexedDB blocking upgrade. Current: ${currentVersion}, Upgrade: ${blockedVersion}. Closing connection.`);
                // Attempt to close the connection in the blocking tab
                // db.close(); // 'db' is not available here, need to handle differently if required
            },
            terminated() {
                // Fires if the browser abruptly terminates the connection (e.g., OS shutdown)
                console.error('IndexedDB connection terminated unexpectedly.');
                dbPromise = null; // Reset promise to allow reconnection attempt
            },
        });
    }
    return dbPromise;
}

// --- List CRUD Operations ---

/** Gets a single list (including items) from IndexedDB by ID. */
export async function getListFromDb(id: number): Promise<ListDetail | undefined> {
    const db = await getDb();
    return db.get('lists', id);
}

/** Gets all lists stored in IndexedDB. */
export async function getAllListsFromDb(): Promise<ListDetail[]> {
    const db = await getDb();
    // Consider adding sorting or filtering here if needed
    return db.getAll('lists');
}

/** Adds or updates a list in IndexedDB. */
export async function putListToDb(list: ListDetail | ListPublic): Promise<number> {
    const db = await getDb();
    // Ensure items array exists, even if empty, for ListDetail type consistency
    const listToStore: ListDetail = {
        ...list,
        items: (list as ListDetail).items ?? [] // Default to empty array if items missing
    };
    return db.put('lists', listToStore);
}

/** Deletes a list and its associated items from IndexedDB. */
export async function deleteListFromDb(id: number): Promise<void> {
    const db = await getDb();
    // Use a transaction to delete list and its items atomically
    const tx = db.transaction(['lists', 'items'], 'readwrite');
    const listStore = tx.objectStore('lists');
    const itemStore = tx.objectStore('items');
    const itemIndex = itemStore.index('listId'); // Use the index

    // Delete the list itself
    await listStore.delete(id);

    // Find and delete all items associated with the list
    let cursor = await itemIndex.openCursor(id.toString()); // Open cursor on the index with the listId
    while (cursor) {
        await cursor.delete(); // Delete the item the cursor points to
        cursor = await cursor.continue(); // Move to the next item with the same listId
    }

    await tx.done; // Complete the transaction
    console.log(`Deleted list ${id} and its items from DB.`);
}

// --- Item CRUD Operations ---

/** Gets a single item from IndexedDB by ID. */
export async function getItemFromDb(id: number): Promise<ItemPublic | undefined> {
    const db = await getDb();
    return db.get('items', id);
}

/** Gets all items for a specific list from IndexedDB using the index. */
export async function getItemsByListIdFromDb(listId: number): Promise<ItemPublic[]> {
    const db = await getDb();
    return db.getAllFromIndex('items', 'listId', listId.toString());
}

/** Adds or updates an item in IndexedDB. */
export async function putItemToDb(item: ItemPublic): Promise<number> {
    const db = await getDb();
    return db.put('items', item);
}

/** Deletes an item from IndexedDB by ID. */
export async function deleteItemFromDb(id: number): Promise<void> {
    const db = await getDb();
    return db.delete('items', id);
}

// --- Sync Queue Operations ---

/** Adds an action to the synchronization queue. */
export async function addSyncAction(action: Omit<SyncAction, 'id'>): Promise<number> {
    const db = await getDb();
    // Add the action (payload should be serializable)
    return db.add('syncQueue', action);
}

/** Retrieves all actions currently in the synchronization queue. */
export async function getSyncQueue(): Promise<SyncAction[]> {
    const db = await getDb();
    // Fetch all items, default order is by key (insertion order)
    return db.getAll('syncQueue');
}

/** Deletes a specific action from the synchronization queue by its ID. */
export async function deleteSyncAction(id: number): Promise<void> {
    const db = await getDb();
    return db.delete('syncQueue', id);
}