// src/lib/syncService.ts
import { browser } from '$app/environment';
import { getSyncQueue, deleteSyncAction } from './db'; // Import DB functions
import { apiClient, ApiClientError } from './apiClient'; // Import API client
import { writable, get } from 'svelte/store'; // Import get for reading store value

// Store for sync status feedback
export const syncStatus = writable<'idle' | 'syncing' | 'error'>('idle');
export const syncError = writable<string | null>(null);

let isSyncing = false; // Prevent concurrent sync runs

/**
 * Processes the offline synchronization queue.
 * Fetches actions from IndexedDB and attempts to send them to the API.
 * Removes successful actions, handles basic errors/conflicts.
 */
export async function processSyncQueue() {
    // Run only in browser, when online, and if not already syncing
    if (!browser || !navigator.onLine || isSyncing) {
        if (isSyncing) console.log('Sync: Already in progress, skipping.');
        return;
    }

    isSyncing = true;
    syncStatus.set('syncing');
    syncError.set(null); // Clear previous errors
    console.log('Sync: Starting queue processing...');

    try {
        const queue = await getSyncQueue();
        console.log(`Sync: Found ${queue.length} actions in queue.`);

        if (queue.length === 0) {
            syncStatus.set('idle');
            isSyncing = false;
            return; // Nothing to do
        }

        // Process actions one by one (sequential processing)
        for (const action of queue) {
            // Should always have an ID from IndexedDB autoIncrement
            if (!action.id) {
                console.error("Sync: Action missing ID, skipping.", action);
                continue;
            }

            console.log(`Sync: Processing action ID ${action.id}, Type: ${action.type}`);
            let success = false;
            try {
                // --- Perform API call based on action type ---
                switch (action.type) {
                    case 'create_list':
                        await apiClient.post('/v1/lists', action.payload);
                        // TODO: Handle mapping tempId if used
                        break;
                    case 'update_list':
                        // Assuming payload is { id: listId, data: ListUpdate }
                        await apiClient.put(`/v1/lists/${action.payload.id}`, action.payload.data);
                        break;
                    case 'delete_list':
                        // Assuming payload is { id: listId }
                        await apiClient.delete(`/v1/lists/${action.payload.id}`);
                        break;
                    case 'create_item':
                        // Assuming payload is { listId: number, data: ItemCreate }
                        await apiClient.post(`/v1/lists/${action.payload.listId}/items`, action.payload.data);
                        // TODO: Handle mapping tempId if used
                        break;
                    case 'update_item':
                        // Assuming payload is { id: itemId, data: ItemUpdate }
                        await apiClient.put(`/v1/items/${action.payload.id}`, action.payload.data);
                        break;
                    case 'delete_item':
                        // Assuming payload is { id: itemId }
                        await apiClient.delete(`/v1/items/${action.payload.id}`);
                        break;
                    default:
                        console.error(`Sync: Unknown action type: ${(action as any).type}`);
                        // Optionally treat as error or just skip
                        throw new Error(`Unknown sync action type: ${(action as any).type}`);
                }

                success = true; // Mark as successful if API call didn't throw
                console.log(`Sync: Action ID ${action.id} (${action.type}) successful.`);
                // Remove from queue ONLY on definite success
                await deleteSyncAction(action.id);

            } catch (err: any) {
                console.error(`Sync: Failed to process action ID ${action.id} (${action.type})`, err);

                // --- Basic Conflict/Error Handling ---
                let errorHandled = false;
                if (err instanceof ApiClientError) {
                    if (err.status === 409) { // Example: Conflict
                        syncError.set(`Sync conflict for ${action.type} (ID: ${action.payload?.id ?? 'N/A'}). Data may be outdated. Please refresh.`);
                        // Remove conflicting action from queue - requires manual refresh/resolution by user
                        await deleteSyncAction(action.id);
                        errorHandled = true;
                    } else if (err.status >= 400 && err.status < 500 && err.status !== 401) {
                        // Other client errors (400 Bad Request, 403 Forbidden, 404 Not Found)
                        // Often mean the action is invalid now (e.g., deleting something already deleted).
                        syncError.set(`Sync failed for ${action.type} (Error ${err.status}). Action discarded.`);
                        await deleteSyncAction(action.id);
                        errorHandled = true;
                    }
                    // Note: 401 Unauthorized is handled globally by apiClient, which calls logout.
                    // Sync might stop if token becomes invalid mid-process.
                }

                if (!errorHandled) {
                    // Network error or Server error (5xx) - Keep in queue and stop processing for now
                    syncError.set(`Sync failed for ${action.type}. Will retry later.`);
                    syncStatus.set('error'); // Indicate sync stopped due to error
                    isSyncing = false; // Allow retry later
                    return; // Stop processing the rest of the queue
                }
            }
        } // End for loop

        // If loop completed without critical errors
        console.log('Sync: Queue processing finished.');
        syncStatus.set('idle'); // Reset status if all processed or handled

    } catch (outerError) {
        // Catch errors during queue fetching or unexpected issues in the loop
        console.error("Sync: Critical error during queue processing loop.", outerError);
        syncError.set("An unexpected error occurred during synchronization.");
        syncStatus.set('error');
    } finally {
        isSyncing = false; // Ensure this is always reset
        // If an error occurred and wasn't handled by stopping, ensure status reflects it
        if (get(syncError) && get(syncStatus) !== 'error') {
            syncStatus.set('error');
        }
    }
}

// --- Initialize Sync ---

// Listen for online event to trigger sync
if (browser) {
    window.addEventListener('online', processSyncQueue);
    // Trigger sync shortly after app loads if online
    if (navigator.onLine) {
        setTimeout(processSyncQueue, 3000); // Delay 3s
    }
}

// Optional: Add function to manually trigger sync if needed from UI
export function triggerSync() {
    console.log("Sync: Manual trigger requested.");
    processSyncQueue();
}