// src/routes/(app)/lists/[listId]/+page.ts
import { error } from '@sveltejs/kit';
import { apiClient, ApiClientError } from '$lib/apiClient';
import type { ListDetail } from '$lib/schemas/list';
import type { ExpenseRecordPublic } from '$lib/schemas/expense'; // Import expense type
import type { PageLoad } from './$types'; // SvelteKit's type for load functions

// Define the expected shape of the data returned by this load function
export interface ListDetailPageLoadData {
    list: ListDetail; // The fetched list data (including items)
    expenses: ExpenseRecordPublic[]; // Array of expense records for the list
    expensesError?: string | null; // Optional error message specifically for expense loading
}

/**
 * Load function for the List Detail page.
 * Fetches both the list details (including items) and the expense history for the list.
 * Handles errors and ensures the user has permission to view the list.
 */
export const load: PageLoad<ListDetailPageLoadData> = async ({ params, fetch }) => {
    // Get listId from the URL parameter provided by SvelteKit routing
    const listId = params.listId;
    console.log(`List Detail page load: Fetching data for list ID: ${listId}`);

    // Validate the listId parameter
    if (!listId || isNaN(parseInt(listId, 10))) {
        console.error("List Detail load: Invalid List ID parameter", listId);
        throw error(400, 'Invalid List ID provided in URL.'); // Use SvelteKit's error helper
    }

    const numericListId = parseInt(listId, 10);

    try {
        // Fetch list details and expense history concurrently for efficiency
        const [listResult, expensesResult] = await Promise.allSettled([
            apiClient.get<ListDetail>(`/v1/lists/${numericListId}`), // Fetch specific list details
            apiClient.get<ExpenseRecordPublic[]>(`/v1/lists/${numericListId}/expenses`) // Fetch expense history
        ]);

        let listData: ListDetail;
        let expensesData: ExpenseRecordPublic[] = [];
        let expensesLoadError: string | null = null;

        // --- Process List Result (Critical) ---
        if (listResult.status === 'fulfilled' && listResult.value) {
            listData = listResult.value;
            console.log(`List Detail load: Successfully fetched list ${numericListId}`);
        } else {
            // Handle list fetch failure - this is critical, so throw SvelteKit error
            const reason = listResult.status === 'rejected' ? listResult.reason : new Error('List data was unexpectedly missing after fetch.');
            console.error(`List Detail page load: Failed to fetch critical list data for ${numericListId}:`, reason);

            if (reason instanceof ApiClientError) {
                // Throw specific SvelteKit errors based on API status code
                if (reason.status === 404) {
                    throw error(404, 'List not found.');
                }
                if (reason.status === 403) {
                    throw error(403, 'Forbidden: You do not have permission to view this list.');
                }
                // Throw a generic server error for other API client issues
                throw error(reason.status || 500, `API Error loading list: ${reason.message}`);
            }
            // Throw a generic 500 error for non-API errors during list fetch
            throw error(500, `Failed to load list data: ${reason instanceof Error ? reason.message : 'Unknown error'}`);
        }

        // --- Process Expenses Result (Non-Critical) ---
        // If fetching expenses fails, we still render the page but show an error message.
        if (expensesResult.status === 'fulfilled' && expensesResult.value) {
            expensesData = expensesResult.value ?? [];
            console.log(`List Detail load: Successfully fetched ${expensesData.length} expense records for list ${numericListId}`);
        } else {
            const reason = expensesResult.status === 'rejected' ? expensesResult.reason : new Error('Expenses data was unexpectedly missing after fetch.');
            console.error(`List Detail page load: Failed to fetch expense history for list ${numericListId}:`, reason);
            // Store the error message to be passed to the page component
            expensesLoadError = `Failed to load expense history: ${reason instanceof Error ? reason.message : 'Unknown error'}`;
            // We don't throw here, allowing the page to render with the list data
        }

        // Return all data needed by the page component
        return {
            list: listData,
            expenses: expensesData,
            expensesError: expensesLoadError
        };

    } catch (err) {
        // Catch errors thrown by the list processing block above or other unexpected errors
        console.error(`List Detail page load: Unexpected error during data fetching for list ${listId}:`, err);

        // Check if it's a SvelteKit error object (already thrown with status/message)
        if (err instanceof Error && 'status' in err && typeof err.status === 'number') {
            throw err; // Re-throw SvelteKit errors
        }

        // Throw a generic 500 error for any other unexpected issues
        throw error(500, `An unexpected error occurred while loading page data: ${err instanceof Error ? err.message : 'Unknown error'}`);
    }
};